@@ -54,6 +54,8 @@
 extern void q1_cmd_or_dir_changed(struct game* g);
 extern void qw_cmd_or_dir_changed(struct game* g);
 extern void q2_cmd_or_dir_changed(struct game* g);
+extern void ut2004_cmd_or_dir_changed(struct game* g);
+extern void doom3_cmd_or_dir_changed(struct game* g);
 
 static struct player *poqs_parse_player(char *tokens[], int num, struct server *s);
 static struct player *qw_parse_player(char *tokens[], int num, struct server *s);
@@ -82,7 +84,9 @@ static int quake_config_is_valid (struct server *s);
 static int quake3_config_is_valid (struct server *s);
 static int config_is_valid_generic (struct server *s);
 
-static int write_quake_variables (const struct condef *con);
+static int write_q1_vars (const struct condef *con);
+static int write_qw_vars (const struct condef *con);
+static int write_q2_vars (const struct condef *con);
 
 static int q1_exec_generic (const struct condef *con, int forkit);
 static int qw_exec (const struct condef *con, int forkit);
@@ -119,7 +123,6 @@ static gboolean quake_has_map(struct server* s);
 static void q3_init_maps(enum server_type);
 static size_t q3_get_mapshot(struct server* s, guchar** buf);
 
-static void doom3_cmd_or_dir_changed(struct game* g);
 static void doom3_init_maps(enum server_type);
 static size_t doom3_get_mapshot(struct server* s, guchar** buf);
 static gboolean doom3_has_map(struct server* s);
@@ -154,8 +157,6 @@ static struct quake_private cod_private;
 static struct quake_private jk3_private;
 static struct quake_private doom3_private;
 
-static void ut2004_cmd_or_dir_changed(struct game* g);
-
 #include "games.c"
 
 struct gsname2type_s
@@ -182,13 +183,12 @@ static struct gsname2type_s gsname2type[] =
 
 void init_games()
 {
-  int i;
+  unsigned i;
 
-  debug(3,"init_games");
+  debug(3,"initializing games");
 
   for (i = 0; i < GAMES_TOTAL; i++)
   {
-    games[i].real_home = expand_tilde (games[i].default_home);
     g_datalist_init(&games[i].games_data);
   }
 
@@ -207,15 +207,6 @@ void init_games()
   				   ("Note:  Savage will not launch correctly without\n"\
     				    "modifications to the game's startup script. Please see the\n"\
 			  	    "XQF documentation for more information.")));
-
-  game_set_attribute(SSAM_SERVER,"game_notes",strdup(_
-  				   ("Note: You need to create a qstat config file for this game to work.\n"\
-    				    "Please see the XQF documentation for more information."))); 
-
-  game_set_attribute(SSAMSE_SERVER,"game_notes",strdup(_
-  				   ("Note: You need to create a qstat config file for this game to work.\n"\
-    				    "Please see the XQF documentation for more information."))); 
-
 }
 
 void games_done()
@@ -1471,6 +1462,15 @@ static void q3_analyze_serverinfo (struct server *s) {
     else if (!strcmp(*info_ptr, "sv_punkbuster") && info_ptr[1] && info_ptr[1][0] == '1') {
       s->flags |= SERVER_PUNKBUSTER;
     }
+    else if (!strcmp(*info_ptr, "protocol")) {
+      const char* masterprotocol = game_get_attribute(s->type,"masterprotocol");
+      
+      if(masterprotocol && !strcmp(masterprotocol, "auto"))
+	masterprotocol = game_get_attribute(s->type, "_masterprotocol");
+
+      if(masterprotocol && strcmp(masterprotocol, info_ptr[1]))
+	s->flags |= SERVER_INCOMPATIBLE;
+    }
   }
 
   if(fs_game)
@@ -1572,25 +1572,11 @@ static int quake_config_is_valid (struct server *s) {
   char *path;
   struct game *g = &games[s->type];
 
-  switch (s->type) {
-
-  case Q1_SERVER:
-    cfgdir = "id1";
-    break;
-
-  case QW_SERVER:
-    cfgdir = (default_qw_is_quakeforge?"base":"id1");
-    break;
-
-  case Q2_SERVER:
-    cfgdir = "baseq2";
-    break;
-
-  default:
+  if(g->main_mod && g->main_mod[0])
+    cfgdir = g->main_mod[0];
+  else
     return FALSE;
 
-  }
-
   if (g->cmd == NULL || g->cmd[0] == '\0') {
     // %s = game name e.g. QuakeWorld
     dialog_ok (NULL, _("%s command line is empty."), g->name);
@@ -1719,20 +1705,29 @@ static int config_is_valid_generic (struct server *s) {
   return TRUE;
 }
 
-
-static FILE *open_cfg (const char *filename, int secure) {
-  FILE *f;
-
-  f = fopen (filename, "w");
-  if (f) {
-    if (secure)
-      chmod (filename, S_IRUSR | S_IWUSR);
-
-    fprintf (f, "//\n// generated by XQF, do not modify\n//\n");
-  }
-  return f;
+static int complain_launch_cant_write_file(const char* file)
+{
+  return dialog_yesno (NULL, 1, _("Launch"), _("Cancel"), 
+	//%s frontend.cfg
+	_("Cannot write to file \"%s\".\n\nLaunch client anyway?"), file);
 }
 
+static FILE *open_cfg (const char *filename, int private) {
+  FILE *f;
+  mode_t saved_umask;
+
+  if(private)
+    saved_umask = umask(0077);
+
+  f = fopen (filename, "w");
+  if (f)
+    fprintf (f, "//\n// generated by XQF, do not modify\n//\n");
+
+  if(private)
+    umask(saved_umask);
+
+  return f;
+}
 
 static int real_password (const char *password) {
   if (!password || (password[0] == '1' && password[1] == '\0'))
@@ -1740,7 +1735,6 @@ static int real_password (const char *password) {
   return TRUE;
 }
 
-
 static int write_passwords (const char *filename, const struct condef *con) {
   FILE *f;
 
@@ -1755,20 +1749,30 @@ static int write_passwords (const char *filename, const struct condef *con) {
     fprintf (f, "spectator \"%s\"\n", con->spectator_password);
 
   if (con->rcon_password)
-    fprintf (f, "rconpassword \"%s\"\n", con->rcon_password);
+    fprintf (f, "rcon_password \"%s\"\n", con->rcon_password);
 
   fclose (f);
   return TRUE;
 }
 
 
-static int write_q1_vars (const char *filename, const struct condef *con) {
+static int write_q1_vars (const struct condef *con) {
   FILE *f;
+  int ret = TRUE;
+  struct game *g = &games[con->s->type];
+  char* file = NULL;
 
-  f = open_cfg (filename, FALSE);
-  if (!f)
+  if(g->main_mod && g->main_mod[0])
+    file = g->main_mod[0];
+  else
     return FALSE;
 
+  file = g_strjoin("/", g->real_dir, file, EXEC_CFG, NULL);
+
+  f = open_cfg (file, FALSE);
+  if (!f)
+    { ret = FALSE; goto out; }
+
   if (default_q1_name)
     fprintf (f, "name \"%s\"\n", default_q1_name);
 
@@ -1781,18 +1785,34 @@ static int write_q1_vars (const char *filename, const struct condef *con) {
     fprintf (f, "exec \"%s\"\n", con->custom_cfg);
 
   fclose (f);
-  return TRUE;
+
+out:
+
+  if(!ret)
+    ret = complain_launch_cant_write_file(file);
+
+  g_free(file);
+  return ret;
 }
 
-
-static int write_qw_vars (const char *filename, const struct condef *con) {
+static int write_qw_vars (const struct condef *con) {
   FILE *f;
   int auto_pl;
+  int ret = TRUE;
+  struct game *g = &games[con->s->type];
+  char* file = NULL;
 
-  f = open_cfg (filename, FALSE);
-  if (!f)
+  if(g->main_mod && g->main_mod[0])
+    file = g->main_mod[0];
+  else
     return FALSE;
 
+  file = g_strjoin("/", g->real_dir, file, EXEC_CFG, NULL);
+
+  f = open_cfg (file, FALSE);
+  if (!f)
+    { ret = FALSE; goto out; }
+
   if (default_qw_name)
     fprintf (f, "name \"%s\"\n", default_qw_name);
 
@@ -1843,17 +1863,33 @@ static int write_qw_vars (const char *filename, const struct condef *con) {
     fprintf (f, "exec \"%s\"\n", con->custom_cfg);
 
   fclose (f);
-  return TRUE;
+
+out:
+
+  if(!ret)
+    ret = complain_launch_cant_write_file(file);
+
+  g_free(file);
+  return ret;
 }
 
-
-static int write_q2_vars (const char *filename, const struct condef *con) {
+static int write_q2_vars (const struct condef *con) {
   FILE *f;
+  int ret = TRUE;
+  struct game *g = &games[con->s->type];
+  char* file = NULL;
 
-  f = open_cfg (filename, FALSE);
-  if (!f)
+  if(g->main_mod && g->main_mod[0])
+    file = g->main_mod[0];
+  else
     return FALSE;
 
+  file = g_strjoin("/", g->real_dir, file, EXEC_CFG, NULL);
+
+  f = open_cfg (file, FALSE);
+  if (!f)
+    { ret = FALSE; goto out; }
+
   if (default_q2_name)
     fprintf (f, "set name \"%s\"\n", default_q2_name);
 
@@ -1872,85 +1908,16 @@ static int write_q2_vars (const char *filename, const struct condef *con) {
     fprintf (f, "exec \"%s\"\n", con->custom_cfg);
 
   fclose (f);
-  return TRUE;
+
+out:
+
+  if(!ret)
+    ret = complain_launch_cant_write_file(file);
+
+  g_free(file);
+  return ret;
 }
 
-/*
-#ifdef QSTAT23
-
-static int write_q3_vars (const char *filename, const struct condef *con) {
-  FILE *f;
-
-  f = open_cfg (filename, FALSE);
-  if (!f)
-    return FALSE;
-
-  if (default_name)
-    fprintf (f, "set name \"%s\"\n", default_name);
-
-  if (games[Q3_SERVER].game_cfg)
-    fprintf (f, "exec \"%s\"\n", games[Q3_SERVER].game_cfg);
-
-  if (con->custom_cfg)
-    fprintf (f, "exec \"%s\"\n", con->custom_cfg);
-
-  fclose (f);
-  return TRUE;
-}
-
-#endif
-*/
-
-static int write_quake_variables (const struct condef *con) {
-/*
-#ifdef QSTAT23
-  char *path;
-#endif
-*/
-  char *file = NULL;
-  int res = TRUE;
-
-  switch (con->s->type) {
-
-  case Q1_SERVER:
-    file = file_in_dir (games[Q1_SERVER].real_dir, "id1/" EXEC_CFG);
-    res =  write_q1_vars (file, con);
-    break;
-
-  case QW_SERVER:
-    if(default_qw_is_quakeforge)
-      file = file_in_dir (games[QW_SERVER].real_dir, "base/" EXEC_CFG);
-    else
-      file = file_in_dir (games[QW_SERVER].real_dir, "id1/" EXEC_CFG);
-    res =  write_qw_vars (file, con);
-    break;
-
-  case Q2_SERVER:
-    file = file_in_dir (games[Q2_SERVER].real_dir, "baseq2/" EXEC_CFG);
-    res = write_q2_vars (file, con);
-    break;
-
-  default:
-    return FALSE;
-
-  }
-
-  if (file) {
-    if (!res) {
-      if (!dialog_yesno (NULL, 1, _("Launch"), _("Cancel"), 
-	    //%s frontend.cfg
-             _("Cannot write to file \"%s\".\n\nLaunch client anyway?"), file)) {
-	g_free (file);
-	return FALSE;
-      }
-    }
-    g_free (file);
-  }
-
-  return TRUE;
-}
-
-
 static int is_dir (const char *dir, const char *gamedir) {
   struct stat stat_buf;
   char *path;
@@ -1964,7 +1931,6 @@ static int is_dir (const char *dir, const char *gamedir) {
   return res;
 }
 
-
 static int q1_exec_generic (const struct condef *con, int forkit) {
   char *argv[32];
   int argi = 0;
@@ -2017,36 +1983,15 @@ static int q1_exec_generic (const struct condef *con, int forkit) {
 }
 
 
-static int qw_exec (const struct condef *con, int forkit) {
-
-  // Not used for Quake2 anymore, but not changed as I can't test it
-  // right now
-
+static int qw_exec (const struct condef *con, int forkit)
+{
   char *argv[32];
   int argi = 0;
-  char *q_passwd;
   char *cmd;
-  char *file;
+  char *file = NULL;
   struct game *g = &games[con->s->type];
   int retval=-1;
-  char *to_free = NULL;
-  
-  switch (con->s->type) {
 
-  case QW_SERVER:
-    if(default_qw_is_quakeforge)
-      q_passwd = "base/" PASSWORD_CFG;
-    else
-      q_passwd = "id1/" PASSWORD_CFG;
-    break;
-
-  case Q2_SERVER:
-    q_passwd = "baseq2/" PASSWORD_CFG;
-    break;
-
-  default:
-    return retval;
-  }
 
   cmd = strdup_strip (g->cmd);
 
@@ -2054,114 +1999,74 @@ static int qw_exec (const struct condef *con, int forkit) {
   while ((argv[argi] = strtok (NULL, delim)) != NULL)
     argi++;
 
-  switch (con->s->type) {
+  if (default_nosound)
+    argv[argi++] = "-nosound";
 
-  case QW_SERVER:
-    if (default_nosound)
-      argv[argi++] = "-nosound";
-
-    if (default_nocdaudio)
-      argv[argi++] = "-nocdaudio";
-
-    if (con->gamedir) {
-      argv[argi++] = "-gamedir";
-      argv[argi++] = con->gamedir;
-    }
-
-    break;
-
-  case Q2_SERVER:
-    if (default_nosound) {
-      argv[argi++] = "+set";
-      argv[argi++] = "s_initsound";
-      argv[argi++] = "0";
-    }
-
-    argv[argi++] = "+set";
-    argv[argi++] = "cd_nocd";
-    argv[argi++] = (default_nocdaudio)? "1" : "0";
-
-    if (con->gamedir) {
-      argv[argi++] = "+set";
-      argv[argi++] = "game";
-      argv[argi++] = con->gamedir;
-    }
-
-    break;
-
-  default:
-    break;
+  if (default_nocdaudio)
+    argv[argi++] = "-nocdaudio";
 
+  if (con->gamedir) {
+    argv[argi++] = "-gamedir";
+    argv[argi++] = con->gamedir;
   }
 
-  if (con->password || con->rcon_password || 
-                                    real_password (con->spectator_password)) {
-    file = file_in_dir (g->real_dir, q_passwd);
+  if (con->password || con->rcon_password
+      || real_password (con->spectator_password)) {
 
-    if (!write_passwords (file, con)) {
-      //passwords file could not be written
-      if (!dialog_yesno (NULL, 1, _("Launch"), _("Cancel"), 
-             _("Cannot write to file \"%s\".\n\nLaunch client anyway?"), file)) {
-	g_free (file);
-	g_free (cmd);
-	return retval;
-      }
-    }
+    if(g->main_mod && g->main_mod[0])
+      file = g_strjoin("/", g->real_dir, g->main_mod[0], PASSWORD_CFG, NULL);
 
-    g_free (file);
+    if (file && !write_passwords (file, con)
+	&& !complain_launch_cant_write_file(file))
+      goto out;
 
     argv[argi++] = "+exec";
     argv[argi++] = PASSWORD_CFG;
   }
   else {
     if (con->spectator_password) {
-	argv[argi++] = "+spectator";
-	argv[argi++] = con->spectator_password;
+      argv[argi++] = "+spectator";
+      argv[argi++] = con->spectator_password;
     }
   }
 
-  if (con->s->type == Q2_SERVER || (con->s->type == QW_SERVER && 
-				    (!con->gamedir || 
-				     g_strcasecmp (con->gamedir, "qw") == 0 ||
-				     !is_dir (g->real_dir, con->gamedir)) ) ) {
-    argv[argi++] = "+exec";
-    argv[argi++] = EXEC_CFG;
-  }
+  argv[argi++] = "+exec";
+  argv[argi++] = EXEC_CFG;
 
   if (con->server) {
-    if (!con->demo || con->s->type == Q2_SERVER) {
-      argv[argi++] = "+connect";
-      argv[argi++] = con->server;
-    }
     if (con->demo) {
       argv[argi++] = "+record";
       argv[argi++] = con->demo;
       if (con->s->type == QW_SERVER)
 	argv[argi++] = con->server;
     }
+    else {
+      argv[argi++] = "+connect";
+      argv[argi++] = con->server;
+    }
   }
 
   argv[argi] = NULL;
 
   retval = client_launch_exec (forkit, g->real_dir, argv, con->s);
 
-  g_free (cmd);
-  if (to_free)
-    g_free (to_free);
+out:
+  g_free(file);
+  g_free(cmd);
+
   return retval;
 }
 
 static int q2_exec (const struct condef *con, int forkit) {
   char *argv[32];
   int argi = 0;
-  char *q_passwd;
   char *cmd;
-  char *file;
+  char *file = NULL;
   struct game *g = &games[con->s->type];
   int retval=-1;
-  char *to_free = NULL;
   
-  q_passwd = "baseq2/" PASSWORD_CFG;
+  if(g->main_mod && g->main_mod[0])
+    file = g_strjoin("/", g->real_dir, g->main_mod[0], PASSWORD_CFG, NULL);
 
   cmd = strdup_strip (g->cmd);
 
@@ -2185,21 +2090,12 @@ static int q2_exec (const struct condef *con, int forkit) {
     argv[argi++] = con->gamedir;
   }
 
-  if (con->password || con->rcon_password || 
-                                    real_password (con->spectator_password)) {
-    file = file_in_dir (g->real_dir, q_passwd);
+  if (con->password || con->rcon_password
+      || real_password (con->spectator_password)) {
 
-    if (!write_passwords (file, con)) {
-      //passwords file could not be written
-      if (!dialog_yesno (NULL, 1, _("Launch"), _("Cancel"), 
-             _("Cannot write to file \"%s\".\n\nLaunch client anyway?"), file)) {
-	g_free (file);
-	g_free (cmd);
-	return retval;
-      }
-    }
-
-    g_free (file);
+    if (file && !write_passwords (file, con)
+	&& !complain_launch_cant_write_file(file))
+      goto out;
 
     argv[argi++] = "+exec";
     argv[argi++] = PASSWORD_CFG;
@@ -2227,9 +2123,9 @@ static int q2_exec (const struct condef *con, int forkit) {
 
   retval = client_launch_exec (forkit, g->real_dir, argv, con->s);
 
+out:
+  g_free (file);
   g_free (cmd);
-  if (to_free)
-    g_free (to_free);
   return retval;
 }
 
@@ -2240,8 +2136,6 @@ static int q2_exec_generic (const struct condef *con, int forkit) {
   struct game *g = &games[con->s->type];
   int retval;
 
-  char *to_free = NULL;
-
   cmd = strdup_strip (g->cmd);
 
   argv[argi++] = strtok (cmd, delim);
@@ -2280,8 +2174,6 @@ static int q2_exec_generic (const struct condef *con, int forkit) {
   retval = client_launch_exec (forkit, g->real_dir, argv, con->s);
 
   g_free (cmd);
-  if (to_free)
-    g_free (to_free);
 
   return retval;
 }
@@ -2573,6 +2465,11 @@ static int hl_exec (const struct condef *con, int forkit) {
   while ((argv[argi] = strtok (NULL, delim)) != NULL)
     argi++;
 
+  if(con->s->type == HL2_SERVER) // XXX
+  {
+    argv[argi++] = "-steam";
+  }
+
   if (con->gamedir) {
     argv[argi++] = "-game";
     argv[argi++] = con->gamedir;
@@ -3399,6 +3296,11 @@ static void quake_init_maps(enum server_type type)
   pd->maphash = q3_init_maphash();
 
   findquakemaps(pd->maphash,games[type].real_dir);
+
+  if(games[type].real_home && access(games[type].real_home, R_OK) == 0)
+  {
+    findquakemaps(pd->maphash,games[type].real_home);
+  }
 }
 
 static void q3_init_maps(enum server_type type)
@@ -3412,11 +3314,9 @@ static void q3_init_maps(enum server_type type)
   pd->maphash = q3_init_maphash();
   findq3maps(pd->maphash,games[type].real_dir);
 
-  if(games[type].real_home)
+  if(games[type].real_home && access(games[type].real_home, R_OK) == 0)
   {
-    char* home = expand_tilde(games[type].real_home);
-    findq3maps(pd->maphash,home);
-    g_free(home);
+    findq3maps(pd->maphash,games[type].real_home);
   }
 }
 
@@ -3431,11 +3331,9 @@ static void doom3_init_maps(enum server_type type)
   pd->maphash = q3_init_maphash();
   finddoom3maps(pd->maphash,games[type].real_dir);
 
-  if(games[type].real_home)
+  if(games[type].real_home && access(games[type].real_home, R_OK) == 0)
   {
-    char* home = expand_tilde(games[type].real_home);
-    finddoom3maps(pd->maphash,home);
-    g_free(home);
+    finddoom3maps(pd->maphash, games[type].real_home);
   }
 }
 
@@ -3449,6 +3347,11 @@ static void unreal_init_maps(enum server_type type)
   ut_clear_maps(pd->maphash);
   pd->maphash = ut_init_maphash();
   findutmaps_dir(pd->maphash,games[type].real_dir,pd->suffix);
+
+  if(games[type].real_home && access(games[type].real_home, R_OK) == 0)
+  {
+    findutmaps_dir(pd->maphash,games[type].real_home,pd->suffix);
+  }
 }
 
 static gboolean quake_has_map(struct server* s)
@@ -3547,74 +3450,4 @@ static gboolean unreal_has_map(struct server* s)
   return ut_lookup_map(hash,s->map);
 }
 
-static void doom3_cmd_or_dir_changed(struct game* g)
-{
-  FILE* f;
-  char* verinfo;
-  const char* attrproto;
-  char line[64];
-
-  debug(3, "cmd: %s, dir: %s", g->cmd, g->real_dir);
-
-  attrproto = game_get_attribute(g->type,"masterprotocol");
-
-  verinfo = file_in_dir (g->real_dir, "version.info");
-  f = fopen(verinfo, "r");
-  if(!f)
-    goto out;
-
-  if(!fgets(line, sizeof(line), f))
-    goto out;
-
-  if(!fgets(line, sizeof(line), f))
-    goto out;
-
-  debug(3, "detected doom3 protocol version %s", line);
-
-  game_set_attribute(g->type, "_masterprotocol", g_strdup(line));
-
-out:
-  if(f) fclose(f);
-  g_free(verinfo);
-}
-
-static void ut2004_cmd_or_dir_changed(struct game* g)
-{
-  FILE* f;
-  char* keyfile = NULL;
-
-  debug(2, "cmd: %s, dir: %s, home: %s", g->cmd, g->real_dir, g->real_home);
-
-  keyfile = file_in_dir (g->real_home, "System/cdkey");
-  if(keyfile)
-  {
-    f = fopen(keyfile, "r");
-    if(!f)
-    {
-      g_free(keyfile);
-      keyfile = NULL;
-    }
-  }
-
-  if(!keyfile)
-  {
-    if(f) fclose(f);
-    keyfile = file_in_dir (g->real_dir, "System/cdkey");
-    if(keyfile)
-    {
-      f = fopen(keyfile, "r");
-      if(!f)
-      {
-	g_free(keyfile);
-	keyfile = NULL;
-      }
-    }
-  }
-
-  if(keyfile || (!keyfile && game_get_attribute(g->type,"cdkey")))
-    game_set_attribute(g->type, "cdkey", keyfile);
-
-  if(f) fclose(f);
-}
-
 // vim: sw=2
