@@ -97,6 +97,9 @@ struct game {
   /** called when either cmd or dir have changed. Also on xqf startup */
   void (*cmd_or_dir_changed)(struct game* g);
 
+  /** sync preference dialog to config */
+  void (*update_prefs)(struct game* g);
+
   /** built in default game specific home directory */
   char *default_home;
 
