@@ -16,13 +16,17 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
+#include "gnuconfig.h"
+
 #include <sys/types.h>
 #include <regex.h>
 #include <string.h>	/* strcmp */
 
 #include <gtk/gtk.h>
 
+#include "i18n.h"
 #include "xqf.h"
+#include "xqf-ui.h"
 #include "srv-list.h"
 #include "dialogs.h"
 #include "sort.h"
@@ -31,14 +35,6 @@
 #include "config.h"
 #include "psearch.h"
 
-#ifdef ENABLE_NLS
-#include <libintl.h>
-#define _(string) gettext(string)
-#define N_(string) (string)
-#else
-#define _(string) (string)
-#endif
-
 #define REGCOMP_FLAGS 	(REG_EXTENDED | REG_NOSUB | REG_ICASE)
 
 
@@ -396,12 +392,15 @@ void find_player (int find_next) {
 
   if (!find_next || server_clist->selection == NULL) {
     dialog_ok (NULL, _("Player not found."));
+    reset_main_status_bar();
   }
   else {
-    if (dialog_yesno (NULL, 0, _("OK"), _("Cancel"),
-          _("End of server list reached.  Continue from beginning?"))) {
+    if (dialog_yesno (_("XQF: End of server list reached"), 0, _("Yes"), _("No"),
+          _("Continue search from beginning?"))) {
       find_player (FALSE);
     }
+    else
+       reset_main_status_bar();
   }
 }
 
