@@ -1068,6 +1068,10 @@ static char *builtin_masters_update_info[] = {
   "ADD HLS,-stm master://steam1.steampowered.com Steam 1",
   "ADD HLS,-stm master://steam2.steampowered.com Steam 2",
 
+  "ADD A2S,-stma2s master://steam1.steampowered.com:27011 Steam 1",
+  "ADD A2S,-stma2s master://steam2.steampowered.com:27011 Steam 2",
+  "ADD A2S,-stma2s master://69.28.151.178:27011 Valve 1",
+
   "ADD Q2S:KP http://www.gameaholic.com/servers/qspy-kingpin Gameaholic.Com",
   "ADD Q2S:KP http://www.ogn.org:6666 OGN",
 
@@ -1087,7 +1091,8 @@ static char *builtin_masters_update_info[] = {
   "ADD T2S master://198.74.35.18:27999 Master 3",
 
 
-  "ADD WOS master://wolf.idsoftware.com:27950 id",
+  "DELETE WOS master://wolf.idsoftware.com:27950 id", // does no longer work
+  "ADD WOS master://wolfmaster.idsoftware.com:27950 id",
   "ADD WOETS master://etmaster.idsoftware.com:27950 id",
 
   "ADD DM3S master://idnet.ua-corp.com:27650 id",
@@ -1129,7 +1134,8 @@ static char *builtin_masters_update_info[] = {
 
   "ADD JK3S master://masterjk3.ravensoft.com:29060 Ravensoft",
 
-  "ADD UT2004S master://ut2004master1.epicgames.com:28902 Epic",
+  "ADD UT2004S master://ut2004master1.epicgames.com:28902 Epic 1",
+  "ADD UT2004S master://ut2004master2.epicgames.com:28902 Epic 2",
   
   "ADD NETP master://netpanzer.dyndns.org netpanzer.dyndns.org",
 
@@ -1163,6 +1169,7 @@ static char *builtin_gslist_masters_update_info[] = {
   "ADD Q2S gslist://master.gamespy.com;gsmtype=quake2 Gslist",
   "ADD Q3S gslist://master.gamespy.com;gsmtype=quake3 Gslist",
   "ADD WOS gslist://master.gamespy.com;gsmtype=rtcw Gslist",
+  "ADD WOETS gslist://master.gamespy.com;gsmtype=rtcwet Gslist",
   "ADD DM3S gslist://master.gamespy.com;gsmtype=doom3 Gslist",
   "ADD RUNESRV gslist://master.gamespy.com;portadjust=-1;gsmtype=rune Gslist",
   "ADD UNS gslist://master.gamespy.com;portadjust=-1;gsmtype=ut Gslist",
