@@ -31,6 +31,10 @@ struct server_props {
   char *server_password;
   char *spectator_password;
   char *rcon_password;
+  int  reserved_slots; /*pulp*/
+
+  int sucks;
+  char *comment;
 };
 
 extern	struct server_props *properties (struct server *s);
@@ -42,7 +46,7 @@ extern	void props_load (void);
 
 extern 	void properties_dialog (struct server *s);
 
-extern	void combo_set_vals (GtkWidget *combo, GList *strlist, char *str);
+extern	void combo_set_vals (GtkWidget *combo, GList *strlist, const char *str);
 
 
 #endif /* __SRV_PROP_H__ */
