@@ -24,8 +24,8 @@
 #include <stdlib.h>	/* strtol */
 #include <unistd.h>	/* read, close, fork, pipe, exec, fcntl, _exit */
                         /* getpid, unlink, write */
-#include <errno.h>      /* errno */
 #include <fcntl.h>	/* fcntl */
+#include <errno.h>      /* errno */
 #include <signal.h>	/* kill, signal... */
 #include <time.h>	/* time */
 #include <sys/socket.h>	/* inet_ntoa */
@@ -333,6 +333,7 @@ static int parse_master_output (char *str, struct stat_conn *conn) {
     g_strchomp (token[n-1]);
 
     if (type != UNKNOWN_SERVER && parse_address (token[n - 1], &addr, &port)) {
+      port += conn->master->options.portadjust;
       h = host_add (addr);
       if (h) {						/* IP address */
 	host_ref (h);
@@ -368,6 +369,7 @@ static int parse_master_output (char *str, struct stat_conn *conn) {
       else {						/* hostname */
 
 	g_strdown (addr);
+	port += conn->master->options.portadjust;
 	if ((us = userver_add (addr, port, type)) != NULL)
 	{
 //	  conn->uservers = userver_list_add (conn->uservers, us);
@@ -702,9 +704,6 @@ static void parse_qstat_record_part2 (GSList *strings, struct server *s) {
 
   s->info = parse_serverinfo (token, n);
 
-  if (games[s->type].analyze_serverinfo)
-    (*games[s->type].analyze_serverinfo) (s);
-
   strings = strings->next;
 
   if (strings && games[s->type].parse_player) {
@@ -721,6 +720,9 @@ static void parse_qstat_record_part2 (GSList *strings, struct server *s) {
     s->players = g_slist_reverse (plist);
   }
 
+  if (games[s->type].analyze_serverinfo)
+    (*games[s->type].analyze_serverinfo) (s);
+
   if(s->type == WO_SERVER || s->type == WOET_SERVER)
   {
     static char* teamnames[2] = { N_("Allies"), N_("Axis") };
@@ -953,18 +955,6 @@ static void stat_servers_input_callback (struct stat_conn *conn, int fd,
   }
 }
 
-
-static void set_nonblock (int fd) {
-  int flags;
-
-  flags = fcntl (fd, F_GETFL, 0);
-  if (flags < 0 || fcntl (fd, F_SETFL, flags | O_NONBLOCK) < 0) {
-    failed ("fcntl", NULL);
-    return;
-  }
-}
-
-
 /**
   return connection to local file
  */
@@ -1050,7 +1040,8 @@ static struct stat_conn *start_qstat (struct stat_job *job, char *argv[],
   if (pid) {	/* parent */
     close (pipefds[1]);
 
-    set_nonblock (pipefds[0]);
+    if(set_nonblock (pipefds[0]) == -1)
+      failed("fcntl", NULL);
 
     conn = g_malloc (sizeof (struct stat_conn));
     conn->buf = g_malloc (BUFFER_MINSIZE);
@@ -1141,6 +1132,7 @@ static struct stat_conn *stat_update_master_qstat (struct stat_job *job,
   struct stat_conn *conn = NULL;
   char *cmd = NULL;
   char *file = NULL;
+  char srcport[12] = {0};
 
   short startprog = 1;
 
@@ -1150,7 +1142,131 @@ static struct stat_conn *stat_update_master_qstat (struct stat_job *job,
 
   if (m->url) {
 
-    if(!strncmp(m->url,master_prefixes[MASTER_FILE],strlen(master_prefixes[MASTER_FILE])))
+    if(m->master_type == MASTER_GSLIST)
+    {
+      int ret = 0;
+      startprog = 1;
+
+      while(current_server_filter > 0 && (cur_filter & FILTER_SERVER_MASK))
+      {
+	struct server_filter_vars* filter =
+	  g_array_index (server_filters, struct server_filter_vars*, current_server_filter-1);
+	size_t bufsize = 2048;
+	char* pos;
+
+	if(!filter)
+	{
+	  xqf_error("filter is NULL");
+	  return NULL;
+	}
+
+	pos = arg_type = g_new0(char, bufsize);
+	if(filter->game_contains&&*filter->game_contains)
+	{
+	  ret = snprintf(pos, bufsize,"(gametype LIKE '%%%s%%')", filter->game_contains);
+	  if(ret == -1)
+	    break;
+	  pos += ret;
+	  bufsize -= ret;
+	}
+
+	if(filter->map_contains&&*filter->map_contains)
+	{
+	  if(pos != arg_type)
+	  {
+	    ret = snprintf(pos, bufsize," AND ");
+	    if(ret == -1)
+	      break;
+	    pos += ret;
+	    bufsize -= ret;
+	  }
+
+	  ret = snprintf(pos, bufsize,"(mapname LIKE '%%%s%%')", filter->map_contains);
+	  if(ret == -1)
+	    break;
+	  pos += ret;
+	  bufsize -= ret;
+	}
+
+	if(filter->server_name_contains&&*filter->server_name_contains)
+	{
+	  if(pos != arg_type)
+	  {
+	    ret = snprintf(pos, bufsize," AND ");
+	    if(ret == -1)
+	      break;
+	    pos += ret;
+	    bufsize -= ret;
+	  }
+
+	  ret = snprintf(pos, bufsize,"(hostname LIKE '%%%s%%')", filter->server_name_contains);
+	  if(ret == -1)
+	    break;
+	  pos += ret;
+	  bufsize -= ret;
+	}
+
+	if(filter->filter_not_empty)
+	{
+	  if(pos != arg_type)
+	  {
+	    ret = snprintf(pos, bufsize," AND ");
+	    if(ret == -1)
+	      break;
+	    pos += ret;
+	    bufsize -= ret;
+	  }
+
+	  ret = snprintf(pos, bufsize,"(numplayers > 0)");
+	  if(ret == -1)
+	    break;
+	  pos += ret;
+	  bufsize -= ret;
+	}
+
+	if(filter->filter_not_full)
+	{
+	  if(pos != arg_type)
+	  {
+	    ret = snprintf(pos, bufsize," AND ");
+	    if(ret == -1)
+	      break;
+	    pos += ret;
+	    bufsize -= ret;
+	  }
+
+	  ret = snprintf(pos, bufsize,"(numplayers < maxplayers)");
+	  if(ret == -1)
+	    break;
+	  pos += ret;
+	  bufsize -= ret;
+	}
+
+	break;
+      }
+
+      if(ret == -1)
+      {
+	g_free(arg_type);
+	arg_type = NULL;
+      }
+
+
+      argv[argi++] = "gslist";
+      argv[argi++] = "-q";
+      argv[argi++] = "-o";
+      argv[argi++] = "5";
+      if(arg_type)
+      {
+	argv[argi++] = "-f";
+	argv[argi++] = arg_type;
+	debug(3, "%s", arg_type);
+      }
+      argv[argi++] = "-N";
+      argv[argi++] = m->options.gsmtype;
+      argv[argi] = NULL;
+    }
+    else if(!strncmp(m->url,master_prefixes[MASTER_FILE],strlen(master_prefixes[MASTER_FILE])))
     {
       startprog = 0;
       file=strdup_strip(m->url + strlen(master_prefixes[MASTER_FILE]));
@@ -1169,7 +1285,7 @@ static struct stat_conn *stat_update_master_qstat (struct stat_job *job,
   }
   else {
     
-    if (m->master_type!=MASTER_LAN && !master_qstat_option(m))
+    if (m->master_type!=MASTER_LAN && m->master_type != MASTER_GAMESPY && !master_qstat_option(m))
       return NULL;
 
     argv[argi++] = QSTAT_EXEC;
@@ -1181,6 +1297,19 @@ static struct stat_conn *stat_update_master_qstat (struct stat_job *job,
 	argv[argi++] = qstat_configfile;
     }
 
+    if(qstat_srcport_low)
+    {
+      snprintf(srcport, sizeof(srcport), "%hu-%hu", qstat_srcport_low, qstat_srcport_high);
+      argv[argi++] = "-srcport";
+      argv[argi++] = srcport;
+    }
+    
+    if(qstat_srcip)
+    {
+      argv[argi++] = "-srcip";
+      argv[argi++] = qstat_srcip;
+    }
+
     argv[argi++] = "-raw";
     argv[argi++] = buf_rawarg;
 
@@ -1193,7 +1322,7 @@ static struct stat_conn *stat_update_master_qstat (struct stat_job *job,
       debug (3, "stat_update_master_qstat() -- MASTER_LAN");
       arg_type = g_strdup_printf("%s,outfile", games[m->type].qstat_option);
     }
-    else if (m->master_type == MASTER_GAMESPY)
+    else if (m->master_type == MASTER_GAMESPY && !master_qstat_option(m))
     {
     	arg_type = g_strdup_printf("-gsm,%s,outfile", games[m->type].qstat_str);
     }
@@ -1201,7 +1330,10 @@ static struct stat_conn *stat_update_master_qstat (struct stat_job *job,
     else if( games[m->type].flags & GAME_QUAKE3_MASTERPROTOCOL )
     {
       // TODO: master protocol should be server specific
-      char* masterprotocol = g_datalist_get_data(&games[m->type].games_data,"masterprotocol");
+      const char* masterprotocol = game_get_attribute(m->type,"masterprotocol");
+      
+      if(masterprotocol && !strcmp(masterprotocol, "auto"))
+	masterprotocol = game_get_attribute(m->type, "_masterprotocol");
 
       if(masterprotocol)
 	arg_type = g_strdup_printf("%s,%s,outfile", master_qstat_option(m),masterprotocol);
@@ -1235,6 +1367,58 @@ static struct stat_conn *stat_update_master_qstat (struct stat_job *job,
 	  filter->filter_not_full?"notfull":"",
 	  NULL);
     }
+    else if( m->type == UT2004_SERVER )
+    {
+      GString* str = NULL;
+      const char* cdkey = game_get_attribute(m->type,"cdkey");
+      if(!cdkey)
+      {
+	xqf_error(_("UT2004 CD Key not found, cannot query master '%s'.\n"
+	      "Make sure the working directory is set correctly."), m->name);
+	goto out;
+      }
+
+      str = g_string_new(NULL);
+      g_string_sprintf(str, "%s,outfile,cdkey=%s", master_qstat_option(m), cdkey);
+
+      if(current_server_filter > 0 && (cur_filter & FILTER_SERVER_MASK))
+      {
+	struct server_filter_vars* filter =
+	  g_array_index (server_filters, struct server_filter_vars*, current_server_filter-1);
+
+	if(filter)
+	{
+	  GString* status = g_string_new(NULL);
+
+	  if(filter->filter_not_empty)
+	    status = g_string_append(status, "notempty");
+
+	  if(filter->filter_not_full)
+	    status = g_string_append(status, "notfull");
+
+	  if(filter->filter_no_password)
+	    status = g_string_append(status, "nopassword");
+
+	  if(filter->game_contains&&*filter->game_contains)
+	  {
+	    g_string_sprintfa(str, ",gametype=%s", filter->game_contains);
+	  }
+
+	  if(status->str && *status->str)
+	  {
+	    g_string_sprintfa(str, ",status=%s", status->str);
+	  }
+	  g_string_free(status, TRUE);
+	}
+      }
+      else
+      {
+	str = g_string_append(str, ",status=nostandard");
+      }
+
+      arg_type = str->str;
+      g_string_free(str, FALSE);
+    }
     else
     {
       arg_type = g_strdup_printf ("%s,outfile", master_qstat_option(m));
@@ -1272,6 +1456,7 @@ static struct stat_conn *stat_update_master_qstat (struct stat_job *job,
     g_free (file);
   }
   
+out:
   g_free (cmd);
   g_free (arg_type);
 
@@ -1290,6 +1475,7 @@ static struct stat_conn *stat_open_conn_qstat (struct stat_job *job) {
   char *fn = NULL;
   GSList *tmp;
   struct stat_conn *conn;
+  char srcport[12] = {0};
 
   if (!job->servers)
     return NULL;
@@ -1312,6 +1498,18 @@ static struct stat_conn *stat_open_conn_qstat (struct stat_job *job) {
     argv[argi++] = qstat_configfile;
   }
 
+  if(qstat_srcport_low)
+  {
+    snprintf(srcport, sizeof(srcport), "%hu-%hu", qstat_srcport_low, qstat_srcport_high);
+    argv[argi++] = "-srcport";
+    argv[argi++] = srcport;
+  }
+  
+  if(qstat_srcip)
+  {
+    argv[argi++] = "-srcip";
+    argv[argi++] = qstat_srcip;
+  }
 
   argv[argi++] = "-maxsimultaneous";
   argv[argi++] = &buf[bufi];
